#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import numpy.testing as npt
import pytest
import pandas as pd
import numpy as np
from cornaconvertor import helpers as hp

dir_path = os.path.dirname(os.path.realpath(__file__))

empty_dataframe = pd.read_excel(os.path.join(dir_path,'lcms_test_data/test_data/Empty xlsx file.xlsx'))
dataframe = pd.read_csv(os.path.join(dir_path,'lcms_test_data/test_data/Victor_input_D2.csv'))
df_missing_col = pd.read_csv(os.path.join(dir_path,'lcms_test_data/test_data/Missing_col.csv'))

def test_check_file_exist():
    with pytest.raises(IOError) as err:
        hp.check_file_exist('path_doesnot_exist.csv')


def test_check_file_exist():
    assert hp.check_file_exist(os.path.join(dir_path,'lcms_test_data/test_data/Victor_input_D2.csv')) == None


def test_check_empty_dataframe():
        assert hp.check_empty_dataframe(dataframe) == None


def test_check_empty_dataframe():
    with pytest.raises(IOError) as err:
        hp.check_empty_dataframe(empty_dataframe)
    assert str(err.value) == 'The dataframe is empty'


def test_get_readfile_func():
    with pytest.raises(IOError) as err:
        hp.get_readfile_func('constants.py')
    assert str(err.value) == 'only csv/xls/xlsx/txt extensions are allowed'


def test_get_readfile_func():
    assert hp.get_readfile_func(os.path.join(dir_path,'lcms_test_data/test_data/Victor_input_D2.csv')) == pd.read_csv


def test_get_readfile_func_error():
    assert hp.get_readfile_func(os.path.join(dir_path,'lcms_test_data/test_data/Empty text file.txt')) == pd.read_table


def test_read_file_error():
    with pytest.raises(IOError) as err:
        hp.read_file(os.path.join(dir_path,'lcms_test_data/test_data/Empty text file.txt'))
    assert str(err.value) == 'The file is empty'


def test_read_file():
    df = pd.DataFrame({'compound': ['betaine', 'betaine'],
                       'note': ['H2-label-1', 'H2-label-3'],
                       'compoundId': ['C00719', 'C00719']})
    read_test_df = pd.read_csv(os.path.join(dir_path, 'lcms_test_data/test_data/test_read_file.csv'))
    npt.assert_array_equal(df, read_test_df)


def test_fetch_chemical_formula_web():
    assert hp.fetch_chemical_formula_web('Glycine') == 'C2H5NO2'


def test_fetch_chemical_formula_web():
    assert hp.fetch_chemical_formula_web('WrongName') == None

def test_check_formula_column_exist():
    df = pd.DataFrame({'compound': ['betaine', 'betaine'],
                       'note': ['H2-label-1', 'H2-label-3'],
                       'formula': ['C00719', 'C00719']})
    assert hp.check_no_formula_column_exist(df) ==False


def test_no_check_label_column_exist():
    df = pd.DataFrame({'compound': ['betaine', 'betaine'],
                       'note': ['H2-label-1', 'H2-label-3'],
                       'label': [np.nan, 'b']})
    assert hp.check_no_label_column_exist(df) == False


def test_check_no_formula_column_exist():
    df = pd.DataFrame({'compound': ['betaine', 'betaine'],
                       'note': ['H2-label-1', 'H2-label-3'],
                       'compoundId': ['C00719', 'C00719']})
    assert hp.check_no_formula_column_exist(df) == True

def test_fetch_chemical_formula():
    assert hp.fetch_chemical_formula('Alpha-ketoisovaleric acid') == 'C5H8O3'


def test_fetch_chemical_formula():
    assert hp.fetch_chemical_formula('Name_not_present') == None


def test_check_columns():
    with pytest.raises(IOError) as err:
        hp.check_columns(df_missing_col) == None
    assert str(err.value) == 'The required columns are missing'


def test_check_column_header():
    test_col_names = ['A', 'B', 'C']
    test_col_header = ['A', 'V', 'B', 'C', 'Z']
    hp.check_column_headers(test_col_header, test_col_names)


def test_check_column_header_wildcard():
    test_col_names = ['A', 'B', 'C']
    test_col_header = ['A', 'V', 'C', 'Z']
    with pytest.raises(AssertionError) as e:
        hp.check_column_headers(test_col_header, test_col_names)
    assert str(e.value) == "Required column/s not found, Column: ['B']"


def test_remove_cit():
    df = pd.DataFrame(columns=["sample"], data=[['abc'],['cde cit'],['xyz'],['jkl cit']])
    df_expected = pd.DataFrame(data=[['abc'],['cde'],['xyz'],['jkl']])
    hp.remove_cit(df, 'sample')
    npt.assert_array_equal(df, df_expected)

def test_write_to_csv():
    input_path = os.path.join(dir_path, 'lcms_test_data/test_data/Empty xlsx file.xlsx')
    output_path = os.path.join(dir_path, 'lcms_test_data/test_data/convert_test.csv')
    assert hp.write_to_csv(input_path, 'convert_test.csv', empty_dataframe) == output_path


def test_rename_columns():
    df = pd.DataFrame(columns=["sample"], data=[['abc'], ['cde cit'], ['xyz'], ['jkl cit']])
    df_expected = pd.DataFrame(columns=["new_sample"], data=[['abc'], ['cde cit'], ['xyz'], ['jkl cit']])
    hp.rename_columns(df, ["sample"], ["new_sample"])
    npt.assert_array_equal(df, df_expected)


def test_get_uniques_values():
    df = pd.DataFrame(columns=["sample"],data=[['abc'],['xyz'],['xyz'],['abc']])
    df_new = hp.get_uniques_values(df, "sample")
    output_expected = np.array(['abc','xyz'])
    npt.assert_array_equal(df_new,output_expected)

def test_filter_col():
    df = pd.DataFrame(columns=["sample"],data=[['abc'],['xyz'],['xyz'],['abc']])
    df_new = hp.filter_col(df, 'sample', 'abc')
    output_expected = pd.DataFrame(columns=["sample"],data=[['abc'],['abc']])
    npt.assert_array_equal(df_new,output_expected)

def test_get_mass_info():
    df = pd.DataFrame({'Component':['Citrate 191/67','Citrate 192/67'],'MASS_INFO':['191.0 / 67.0','192.0 / 67.0']})
    comp_path = os.path.dirname(dir_path)
    comp_path = os.path.join(comp_path,'cornaconvertor')
    df_new = hp.get_mass_info(comp_path)[:2]
    npt.assert_array_equal(df, df_new.sort_index(axis=1))

def test_get_method_type():
    df = pd.DataFrame({'Component Name':['Citrate 191/191','Citrate 191/111'],'Method':['cit','cit']})
    meta_path = os.path.dirname(dir_path)
    meta_path = os.path.join(meta_path, 'cornaconvertor')
    df_new = hp.get_method_type(meta_path)[:2]
    npt.assert_array_equal(df, df_new)

def test_add_missing_comp():
    input_df = pd.DataFrame({'Component Name':['A1','A2','A3','A4'],
                             'Original Filename':['P','Q','P','Q'],
                             'Area':['30','40','50','60'],
                             'Method':['cit','cit','cit','cit']})
    input_df = input_df[['Component Name', 'Original Filename', 'Area', 'Method']]
    expected_df = pd.DataFrame({'Component Name':['A1','A3','A2','A4','A2','A4','A1','A3'],
                                'Original Filename':['P','P','P','P','Q','Q','Q','Q'],
                                'Area':['30','50','0.0','0.0','40','60','0.0','0.0'],
                                'Method':['cit','cit','cit','cit','cit','cit','cit','cit']})

    df_new = hp.add_missing_comp(input_df,'cit')
    npt.assert_array_equal(expected_df, df_new)

testdata = [
    (pd.DataFrame(), [1,2], [1, 2]),
    (pd.DataFrame({'id': [1, 2, 3, 4, 5, 6, 7, 8, 9],
                   'hmdb_id': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i']}),
                    [2, 3, 4, 5, 13, 6, 7, 12] , ['b', 'c', 'd', 'e', 13, 'f', 'g', 12]),
    (pd.DataFrame({'id': [1, 2, 3],
                   'hmdb_id': ['a', 'b', 'c']}), [], [])
    ]

@pytest.mark.parametrize("df, kegg_list, expected", testdata)
def test_replace_kegg_with_hmdb(df, kegg_list, expected):
    """Test to check if hmdb id is present in df then kegg id is replaced by
    hmdbid """
    output = hp.replace_kegg_with_hmdb(df, kegg_list)
    assert output == expected


def test_convert_primary_to_sec_hmdb():
    df = pd.DataFrame({'compoundID': ['HMDB0000094', 'HMDB0000121']})
    df['compoundID'] = df['compoundID'].apply(hp.convert_primary_to_sec_hmdb)
    assert df.size == 2


def test_get_sec_hmdb_df():
    good_peak_df = pd.DataFrame({'compoundId': ['HMDB00094', 'HMDB0000134',
                                           'HMDB0000121', 'HMDB003125']})
    sec_hmdb_df = hp.get_sec_hmdb_df(good_peak_df)
    expected_df = pd.DataFrame({'compoundId': ['HMDB00094', 'HMDB00134',
                                           'HMDB00121', 'HMDB003125']})
    assert sec_hmdb_df.size == expected_df.size

